package cn.sh.stc.sict.cloud.common.gateway.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.web.cors.reactive.CorsUtils;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.WebFilter;
import org.springframework.web.server.WebFilterChain;
import reactor.core.publisher.Mono;

import java.util.HashSet;
import java.util.Set;

/**
 * @Description
 * @Author
 * @Date
 */
@Configuration
public class GlobalCorsConfig {

    /**
     * 这里为支持的请求头，如果有自定义的header字段请自己添加（不知道为什么不能使用*）
     */
    private static final String ALLOWED_HEADERS = "x-requested-with, authorization, Content-Type, Authorization, credential, X-XSRF-TOKEN,token,username,client,access-token";
    private static final String ALLOWED_METHODS = "*";
    private static final Set<String> ALLOWED_ORIGINS = new HashSet<String>(){{
        add("https://inno.sh-sict.com");
        add("https://hy.hpwjsns.org.cn");
        add("http://localhost:12998");
        add("http://127.0.0.1:12998");
        add("http://sict-gateway:12998");
        add("http://192.168.31.140:12998");
        add("http://localhost:12999");
        add("http://127.0.0.1:12999");
        add("http://sict-gateway:12999");
    }};
    private static final String ALLOWED_Expose = "*";
    private static final String MAX_AGE = "18000L";

    @Bean
    public WebFilter corsFilter() {
        return (ServerWebExchange ctx, WebFilterChain chain) -> {
            ServerHttpRequest request = ctx.getRequest();
            if (CorsUtils.isCorsRequest(request)) {
                ServerHttpResponse response = ctx.getResponse();
                HttpHeaders headers = response.getHeaders();
                if(ALLOWED_ORIGINS.contains(request.getHeaders().getOrigin())){
                    headers.add("Access-Control-Allow-Origin", request.getHeaders().getOrigin());
                }else{
                    headers.add("Access-Control-Allow-Origin", "https://hy.hpwjsns.org.cn");
                }
                headers.add("Access-Control-Allow-Methods", ALLOWED_METHODS);
                headers.add("Access-Control-Max-Age", MAX_AGE);
                headers.add("Access-Control-Allow-Headers", ALLOWED_HEADERS);
                headers.add("Access-Control-Expose-Headers", ALLOWED_Expose);
                headers.add("Access-Control-Allow-Credentials", "true");
                if (request.getMethod() == HttpMethod.OPTIONS) {
                    response.setStatusCode(HttpStatus.OK);
                    return Mono.empty();
                }
            }
            return chain.filter(ctx);
        };
    }
}
